import copy
import urllib.parse
import re
import unidecode
import os

import flask
import flask_babel
from flask_babel import lazy_gettext
import jinja2.exceptions

# Base de données des cartes
import gzip
import json
if (os.path.isfile("barrins_codex/library.json.gz")):
	# File exists
	cartes = json.load(gzip.open("barrins_codex/library.json.gz"))
else:
	# Build file
	from . import build_library
	cartes = build_library.build()
library = {}

from . import config
from . import navigation


app = flask.Flask(__name__, template_folder="templates", static_folder='static')
app.jinja_env.policies["ext.i18n.trimmed"] = True
babel = flask_babel.Babel(app)
config.configure_app(app)


def main():
	app.run()

# Adding symbols to context_processor
BASE_CONTEXT = {
	"W": flask.Markup("🔆"),
	"U": flask.Markup("💧"),
	"B": flask.Markup("💀"),
	"R": flask.Markup("🔥"),
	"G": flask.Markup("🌳"),
}

# Retrieving locale and timezone information
@babel.localeselector
def get_locale():
	return flask.g.get("lang_code", app.config["BABEL_DEFAULT_LOCALE"])


@babel.timezoneselector
def get_timezone():
	user = flask.g.get("user", None)
	if user is not None:
		return user.timezone


# Defining Errors
@app.errorhandler(jinja2.exceptions.TemplateNotFound)
@app.errorhandler(404)
def page_not_found(error):
	return flask.render_template("404.html"), 404


# Static file redirection
@app.route("/favicon.ico")
def favicon():
	return flask.redirect(flask.url_for("static", filename="img/favicon.ico"))


# Serve robots and sitemap static files
@app.route('/robots.txt')
@app.route('/sitemap.xml')
def static_from_root():
	return flask.send_from_directory(app.static_folder, flask.request.path[1:])


# Default route
@app.route("/")
@app.route("/<path:page>")
@app.route("/<lang_code>/<path:page>")
def index(lang_code=None, page=None):
	redirect = False
	if not page:
		page = "index.html"
		redirect = True
	if not lang_code or lang_code not in app.config["SUPPORTED_LANGUAGES"].keys():
		if lang_code:
			page = lang_code + "/" + page
		lang_code = flask.request.accept_languages.best_match(
			app.config["SUPPORTED_LANGUAGES"].keys()
		)
		page = "/" + lang_code + "/" + page
		redirect = True
	if redirect:
		return flask.redirect(page, 301)

	flask.g.lang_code = lang_code
	context = copy.copy(BASE_CONTEXT)
	context["lang"] = lang_code
	return flask.render_template(page, **context)


def _i18n_url(page, _anchor=None, locale=None, **params):
	url = "/" + (locale or get_locale()) + page.url
	if params:
		url += "?" + urllib.parse.urlencode(params)
	if _anchor:
		url += "#" + _anchor
	return url


def _link(page, name=None, _anchor=None, _class=None, locale=None, **params):
	if not page or not page.url:
		return ""
	name = name or page.name
	url = _i18n_url(page, _anchor, locale, **params)
	if _class:
		_class = f"class={_class} "
	else:
		_class = ""
	return flask.Markup(f'<a {_class}href="{url}">{name}</a>')


@app.context_processor
def linker():
	path = flask.request.path
	if path[1:3] in app.config["SUPPORTED_LANGUAGES"].keys():
		path = path[3:]
	if path[-11:] == "/index.html":
		path = path[:-11]
	if path[-5:] == ".html":
		path = path[:-5]
	if path[-1:] == "/":
		path = path[:-1]

	def i18n_url(page, _anchor=None, **params):
		return _i18n_url(
			navigation.HELPER.get(page, {}).get("self"), _anchor=_anchor, **params
		)

	def link(page, name=None, _anchor=None, **params):
		return _link(
			navigation.HELPER.get(page, {}).get("self"),
			name=name,
			_anchor=_anchor,
			**params,
		)

	def translation(locale, name):
		return _link(
			navigation.HELPER.get(path, {}).get("self"), name=name, locale=locale
		)

	def top():
		return _link(navigation.HELPER.get(path, {}).get("top"))

	def next():
		return _link(navigation.HELPER.get(path, {}).get("next"), _class="next")

	def prev():
		return _link(navigation.HELPER.get(path, {}).get("prev"), _class="prev")

	def external(url, name):
		return flask.Markup(f'<a target="_blank" href="{url}">{name}</a>')

	return dict(
		i18n_url=i18n_url,
		link=link,
		translation=translation,
		top=top,
		next=next,
		prev=prev,
		external=external,
	)


# reecriture en variables
def _var_name(name):
	name = unidecode.unidecode(name).lower()
	name = re.sub(r"'", "", name)
	name = re.sub(r",", "", name)
	name = re.sub(r"[^a-zA-Z0-9]", "_", name)
	return name


def _name(name):
	name = unidecode.unidecode(name).lower()
	name = re.sub(r"[^a-zA-Z0-9]", "", name)
	return name


def scryfall_id(name):
	name = _name(name)
	return library[name]["id"]

@app.context_processor
def display_card():
	def card(name, display_name=None):
		return flask.Markup(
			"""<span class="card" data-tippy-content="
					<div class='card-container'>
						<img data-src='https://api.scryfall.com/cards/{scryfallId}?format=image' class='card-image'>
					</div>"
				scryfallId="{scryfallId}" >{name}</span>""".format(
				# replace spaces with non-breakable spaces in card names
				name=(display_name or name).replace(" ", " "),
				scryfallId=scryfall_id(name),
			)
		)

	for carte in cartes:
		# Enabling card names checks
		library[list(carte)[0]] = carte[list(carte)[0]]
		# Adding card to context
		BASE_CONTEXT[_var_name(library[list(carte)[0]]['name'])] = card(library[list(carte)[0]]['name'])


	def card_image(name, version="png"):
		img = """
			<img src="https://api.scryfall.com/cards/{scryfallId}?format=image&version={version}"
			alt="{name}" scryfallId="{scryfallId}" />
		"""

		return flask.Markup(
			img.format(
				name=name.replace(" ", " "),
				scryfallId=scryfall_id(name),
				version=version,
			)
		)

	def card_art(name):
		return flask.Markup(
			"""<img src="https://api.scryfall.com/cards/{scryfallId}?format=image&version=art_crop"
			alt="{name}" />""".format(
				name=name.replace(" ", " "),
				scryfallId=scryfall_id(name)
			)
		)

	return dict(card=card, card_image=card_image, card_art=card_art)



# Fichier en paramètre
import os
import sys

# Base de données des cartes
import gzip
import json
library = {}
# Vérification de la présence de la table de correspondance
if (os.path.isfile("barrins_codex/library.json.gz")):
	# File exists === dev
	cartes = json.load(gzip.open("barrins_codex/library.json.gz"))
else:
	# Build file === prod
	import build_library
	cartes = build_library.build()
# Utilisation du fichier
for carte in cartes:
	library[list(carte)[0]] = carte[list(carte)[0]]


# Traitement de texte
import re
import unidecode
def _name(name):
	name = unidecode.unidecode(name).lower()
	name = re.sub(r"[^a-zA-Z]", "", name)
	return name


# Ajout dans un réceptacle
def _get(ligne):
	qte = re.sub(r"[a-zA-Z]","",ligne.split(r" ")[0])
	info = library[_name(ligne)]
	return {
		"count":qte,
		"name":re.split("/",info["name"])[0],
		"id":info["id"],
		"types":info["types"]
	}


# Réceptacles
import json
czon = []
crea = { "type": "Creatures", "count": 0, "cards": [] }
plan = { "type": "Planeswalkers", "count": 0, "cards": [] }
arti = { "type": "Artifacts", "count": 0, "cards": [] }
ench = { "type": "Enchantments", "count": 0, "cards": [] }
inst = { "type": "Instants", "count": 0, "cards": [] }
sorc = { "type": "Sorceries", "count": 0, "cards": [] }
land = { "type": "Lands", "count": 0, "cards": [] }


fn = sys.argv[1]
if os.path.exists(fn):
	# Le fichier existe
	print(os.path.basename(fn))
	# Ouverture du fichier
	f = open(fn, "r")
	# Lecture ligne à ligne
	lignes = f.readlines()
	for ligne in lignes:
		if ligne.startswith("SB:"):
			# Command Zone
			czon.append(_get(ligne[3:]))

		name = _name(re.split("/",ligne)[0])
		if name in library:
			ajout = _get(ligne)
			# First lands because of some specific cards
			if "Land" in ajout["types"]:
				# Land first in case of Dryad Arbor
				if "Creature" in ajout["types"] and "/" not in ligne:
					# Dryad Arbor-like case
					land["cards"].append(ajout)
					land["count"] = land["count"] + int(ajout["count"]) # Increase land count
				if len(ajout["types"]) == 1:
					# Usual land cards
					land["cards"].append(ajout)
					land["count"] = land["count"] + int(ajout["count"]) # Increase land count

			if "Creature" in ajout["types"]:
				if "Land" not in ajout["types"]:
					# usual case
					crea["count"] = crea["count"] + int(ajout["count"])
					crea["cards"].append(ajout)
				if "Land" in ajout["types"] and "/" in ligne:
					# MDFC creature-land case
					crea["count"] = crea["count"] + int(ajout["count"])
					crea["cards"].append(ajout)

			elif "Planeswalker" in ajout["types"]:
				plan["count"] = plan["count"] + int(ajout["count"])
				plan["cards"].append(ajout)

			elif "Artifact" in ajout["types"]:
				arti["count"] = arti["count"] + int(ajout["count"])
				arti["cards"].append(ajout)

			elif "Enchantment" in ajout["types"]:
				ench["count"] = ench["count"] + int(ajout["count"])
				ench["cards"].append(ajout)

			elif "Instant" in ajout["types"]:
				inst["count"] = inst["count"] + int(ajout["count"])
				inst["cards"].append(ajout)

			elif "Sorcery" in ajout["types"]:
				sorc["count"] = sorc["count"] + int(ajout["count"])
				sorc["cards"].append(ajout)

	# Fermeture du fichier
	f.close()

	# Affichage du tableau
	table = {
		"library": {
			"cards": [
				crea,
				plan,
				arti,
				ench,
				inst,
				sorc,
				land
			],
			"count": (100-len(czon))
		},
		"command": {
			"cards":czon,
			"count":len(czon)
		}
	}
	print(json.dumps(table, indent=4, sort_keys=True))# Prettyprint
	#print(json.dumps(table))
	"""
	table = {"library":{"cards":deck,"count":len(deck)},"command":{"cards":czon,"count":len(czon)}}
	print(json.dumps(table))
	"""



import os


class BaseConfig(object):
	DEBUG = os.environ.get("DEBUG", False)
	TESTING = DEBUG
	# Translation
	SUPPORTED_LANGUAGES = {"en": "English", "fr": "Francais"}
	BABEL_DEFAULT_LOCALE = "fr"
	BABEL_DEFAULT_TIMEZONE = "UTC"


def configure_app(app):
	app.config.from_object(BaseConfig)



import setuptools
import setuptools.command.develop
import setuptools.command.install


class WithCompile:
	def run(self):
		super().run()


class InstallWithCompile(WithCompile, setuptools.command.install.install):
	pass


class DevelopWithCompile(WithCompile, setuptools.command.develop.develop):
	pass


class SDistWithCompile(WithCompile, setuptools.command.sdist.sdist):
	pass


setuptools.setup(
	cmdclass={
		"install": InstallWithCompile,
		"develop": DevelopWithCompile,
		"sdist": SDistWithCompile,
	}
)



import collections
import re
import unidecode
from flask_babel import lazy_gettext


Page = collections.namedtuple("Page", ["name", "path", "url"])


class Nav:
	def __init__(self, name, index=False, children=None):
		self.name = name
		self.index = index
		self.children = children or []

	def page(self, path):
		res = unidecode.unidecode(str(self.name))
		res = re.sub(r"[^\sa-zA-Z0-9]", "", res).lower().strip()
		res = re.sub(r"\s+", "-", res)
		if res == "home":
			res = path
		else:
			res = path + "/" + res
		if not self.children:
			url = res + ".html"
		elif self.index:
			url = res + "/index.html"
		else:
			url = None
		return Page(self.name, res, url)

	def walk(self, path=None, top=None, ante=None, post=None):
		page = self.page(path or "")
		if not self.children or self.index:
			yield (page.path, {"self": page, "top": top, "prev": ante, "next": post})
		if self.index:
			top = page
		for i, child in enumerate(self.children):
			if i > 0:
				ante = self.children[i - 1].page(page.path)
			else:
				ante = None
			if i < len(self.children) - 1:
				post = self.children[i + 1].page(page.path)
			else:
				post = None
			yield from child.walk(path=page.path, top=top, ante=ante, post=post)


STRUCTURE = Nav(
	# TRANSLATORS: please abide by MTG translation choices for game terms when possible.
	lazy_gettext("Home"),
	index=True,
	children=[
		Nav(
			lazy_gettext("Articles"),
			index=True,
			children=[
				Nav(
					lazy_gettext("Classifier un deck"),
					index=True,
					children=[
						Nav(lazy_gettext("Les macrotypes")),
						Nav(lazy_gettext("Les archétypes")),
						Nav(lazy_gettext("Les tiers")),
						Nav(lazy_gettext("Pour aller plus loin")),
					],
				),
				Nav(
					lazy_gettext("Complexité et Machine de Turing"),
					index=True,
					children=[
						Nav(lazy_gettext("Jeu, Complexité et Résolution")),
						Nav(lazy_gettext("Magic et Complexité")),
					],
				),
				Nav(lazy_gettext("Construire un deck")),
				Nav(lazy_gettext("Mélanger un deck")),
			],
		),
		Nav(
			lazy_gettext("Archétypes"),
			index=True,
			children=[
				Nav(lazy_gettext("Adeliz, the Cinder Wind")),
				Nav(lazy_gettext("Aminatou, the Fateshifter")),
				Nav(lazy_gettext("Isamaru, Hound of Konda")),
				Nav(lazy_gettext("Golos, Tireless Pilgrim")),
				Nav(lazy_gettext("Kelsien, the Plague")),
				Nav(lazy_gettext("Kess, Dissident Mage")),
				Nav(lazy_gettext("Orvar, the All-Form")),
				Nav(lazy_gettext("Rowan and Will Kenrith")),
				Nav(lazy_gettext("The Gitrog Monster")),
				Nav(lazy_gettext("Titania, Protector of Argoth")),
				Nav(lazy_gettext("Venser, Shaper Savant")),
				Nav(lazy_gettext("Winota, Joiner of Forces")),
			],
		),
		Nav(
			lazy_gettext("Cartes thématiques"),
			index=True,
			children=[
				Nav(
					lazy_gettext("Mécaniques de la colorpie"),
					index=False,
					children=[
						Nav(lazy_gettext("La Destruction de permanents")),
						Nav(lazy_gettext("Les Cartes de taxe")),
						Nav(lazy_gettext("La Protection")),
						Nav(lazy_gettext("Les Contresorts")),
						Nav(lazy_gettext("Le Bounce de permanents")),
						Nav(lazy_gettext("Les Cantrips")),
						Nav(lazy_gettext("La Défausse")),
						Nav(lazy_gettext("La Gestion de créatures")),
						Nav(lazy_gettext("La Réanimation de créatures")),
						Nav(lazy_gettext("Les Dégâts directs")),
						Nav(lazy_gettext("Les Créatures agressives")),
						Nav(lazy_gettext("Le Remplacement de cartes")),
						Nav(lazy_gettext("Le Ramp")),
						Nav(lazy_gettext("Les Mana dorks")),
						Nav(lazy_gettext("Les Grosses Créatures")),
					],
				),
				Nav(lazy_gettext("Les Cantrips")),
				Nav(lazy_gettext("Les Piocheurs")),
				Nav(lazy_gettext("Les Spot Removals")),
				Nav(lazy_gettext("Les Mass Removals")),
			],
		),
	],
)


HELPER = dict(STRUCTURE.walk())



import gzip
import json
import os
import sys
import unidecode
import re
# File Download
import requests
from tqdm.auto import tqdm


def _name(card,name=None):
	name = (name or card["name"])
	name = unidecode.unidecode(name).lower()
	name = re.sub(r"[^a-zA-Z0-9]", "", name)
	return name


def _get(card,set,name=None):
	return {
		"name":(name or card["name"]),
		"date":set["releaseDate"],
		"id":card["identifiers"]["scryfallId"],
		"types":card["types"],
	}


def _download(url):
	file = url.split('/')[-1]
	response = requests.get(url, stream=True)
	with open(file, "wb") as file:
		response = requests.get(url)
		# write to file
		file.write(response.content)


def _delete(file):
	if (os.path.isfile(file)):
		os.remove(file)
	else:
		print(r"{file} not found")


DIR_NAME = "barrins_codex"
SKIP_TYPES = {"from_the_vault","masterpiece","promo","duel_deck","premium_deck","spellbook","token","master","memorabilia","funny"}

# https://api.scryfall.com/cards/{scryfallId}?format=image

def build():
	# Download AllPrintings.json.gz
	_download("https://mtgjson.com/api/v5/AllPrintings.json.gz")
	# Download AllPrintings.json.gz
	_download("https://mtgjson.com/api/v5/SetList.json.gz")

	prints = json.load(gzip.open("AllPrintings.json.gz"))
	sets = json.load(gzip.open("SetList.json.gz"))

	library = {}
	for set in sets["data"]:
		if not set["isOnlineOnly"]:
			if set["type"] not in SKIP_TYPES:
				if len(set["code"]) == 3:
					# Only regular sets
					for card in prints["data"][set["code"]]["cards"]:
						# Adding it to library
						if _name(card) not in library:
							library[_name(card)] = {_name(card):_get(card, set)}
						if _name(card) in library:
							# If newer version exists
							if set["releaseDate"] > library[_name(card)][_name(card)]["date"]:
								library[_name(card)] = {_name(card):_get(card, set)}
						# Cards may have another face : name_face_a / name_face_b
						# Cards with other faces have a "//" separator
						if "//" in card["name"]:
							# Specific handling
							card_faces = card["name"].split(" // ")
							face_a = card_faces[0]
							face_b = card_faces[1]
							# Cards are already in the library
							if (_name(card, face_a) and _name(card, face_b)) in library:
								# Need to check types for any new type
								list_types = library[_name(card)][_name(card)]["types"]
								# library[_name(card)] if used above and will be updated
								for t in card["types"]:
									if t not in list_types:
										list_types.append(t)
								# Adding full set of types to card and all faces
								try:
									library[_name(card, face_a)][_name(card, face_a)]["types"] = list_types
								except KeyError:
									pass
								library[_name(card, face_b)][_name(card, face_b)]["types"] = list_types
							# Adding Face A to library
							if _name(card, face_a) not in library:
								library[_name(card, face_a)] = {_name(card, face_a):_get(card, set, face_a)}
							# Adding Face B to library
							if _name(card, face_b) not in library:
								library[_name(card, face_b)] = {_name(card, face_b):_get(card, set, face_b)}

	try:
		# Generating a file on dev
		if not os.path.isdir(DIR_NAME):
			os.mkdir(DIR_NAME)
		fname = "library.json.gz"
		fpath = os.path.join(DIR_NAME, fname)
		json.dump(list(library.values()), gzip.open(fpath, "wt"))
	except IOError:
		# No generation on prod
		pass

	# Deleting downloaded files
	_delete("AllPrintings.json.gz")
	_delete("SetList.json.gz")

	# Returning constructed library
	return library.values()

if __name__ == "__main__":
	build()
